%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%         Face Recognition using Joint Sparsity Models
%
% Description: Generate the JSM data -like measurement for the given set of 
% images according to their labelled class using the specified parameters.
% Also the measurement matrix Phi, Basis Matrix etc are generated & stored.
%
%  Input/Output 
%  Options: Input data structure containing various parameter settings.  
%  Refer to FR_Document.doc for details.
%
%  SampleFraction: %age of measurement used. Right now this is ignored and
%  for training 100% measurement is used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%              Generate the JSM data 

function generate_JSM_Face_data(Options, SampleFraction)

N1 = Options.SizeImage(1);
N2 = Options.SizeImage(2);
NumImages = Options.M(1);
NumClasses = Options.L;

N = N1*N2;
K = round(SampleFraction*N1*N2);    
xs = zeros(N1*NumImages,N2);
xs_m_sub = zeros(N1*NumImages,N2);
Phi = zeros(N1*NumImages,N2*NumImages);
shi_tilda = zeros(N1*NumImages,(N2+1)*NumImages)
tet = zeros(N1*NumImages,(N2+1)*NumImages);
mean_xs = 0;

%%  Calculate Measurements of mean subtracted images and stack them. 

Opts = Options;
Opts.SampleFraction = SampleFraction;

for i=1:NumClasses
    Opts.ClassNum = i;
    Options.ClassNum = i;
    
    NumImages = Options.M(i);
    for j=1:NumImages
        Opts.index = j;
        im_name = GetPath_name('Image Data', Opts);
        x1 = double(imread(im_name));
        
        if(size(x1,3)== 3)
            grayscale(im_name,im_name);
            x1 = imread(im_name);
        end
        x1 = double(x1)/255;
        mean_xs = mean(mean(x1));
        x1 = x1-mean_xs;
        
        if (0 == Options.twoDReconstruction)
            x1 = x1(:);
        end
        xs((j-1)*N1+1:j*N1,1:N2) = x1;
    end

    xs_m_sub = xs;
    
    if (0 == Options.twoDReconstruction)
        [Phi shi_tilda] = Phi_Shi_TetaMat(Options);
    else
        [Phi shi_tilda] = Phi_Shi_TetaMat2D(Options);
    end        

    y = Phi * xs;
    y_ms = Phi * xs_m_sub;
    tet = Phi * shi_tilda;
    clear shi_tilda;
    
    clear xs;
    clear xs_m_sub;

    if i == NumClasses
        clear Phi;
    end

    Opts.SampleFraction = SampleFraction;
    Opts.NumImages = NumImages;
    Opts.ClassNum = i;
    dt_path1 = GetPath_name('JSM Data Save',Opts);
    save(dt_path1, 'y_ms','tet', 'mean_xs');
end