%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%         Face Recognition using Joint Sparsity Models
%
% Description: Converts the 2D basis matrix as 1D basis matrix, so that the
% 2D transform can be calculated on a vectored signal. As an example,  
% consider the 2D DCT transform - Y = C'*y*C where, y is a 2D signal and C is
% the DCT matrix Y is the transform coeff 2D signal. The same can be
% achieved as Y(:) = Ck*y(:) where Ck = khronical_product(C) 
%
%  Input/Output 
%  x: Input Orthonormal 2D basis matrix
%  
%  X: Output 1D equivelent basis matrix. 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function X = khronical_product(x)

[N1 N2]= size(x);

for n1=1:N1
    for n2=1:N2
        X( ((n1-1)*N1+1):((n1-1)*N1+N1),((n2-1)*N2+1):((n2-1)*N2+N2) ) = x(n1,n2).*x;
    end
end